@@ -9,6 +9,7 @@ package inc::MBX::FLTK;
     use File::Find qw[find];
     use File::Path qw[make_path];
     use base 'Module::Build';
+    our %dl_funcs;
     {
 
         package My::ExtUtils::CBuilder;
@@ -26,11 +27,33 @@ package inc::MBX::FLTK;
             #(my( $program), @cmd) = @cmd;
             #return !system ($program, @cmd);
         }
+
+        sub prelink {
+            my ($self, %args) = @_;
+            ($args{dl_file} = $args{dl_name}) =~ s/.*:://
+                unless $args{dl_file};
+            $args{'dl_funcs'} = \%inc::MBX::FLTK::dl_funcs; # Frakin' CBuilder
+            require ExtUtils::Mksymlists;
+            ExtUtils::Mksymlists::Mksymlists( # dl. abbrev for dynamic library
+                DL_VARS  => $args{dl_vars}      || [],
+                DL_FUNCS => $args{dl_funcs}     || {},
+                FUNCLIST => $args{dl_func_list} || [],
+                IMPORTS  => $args{dl_imports}   || {},
+                NAME   => $args{dl_name},     # Name of the Perl module
+                DLBASE => $args{dl_base},     # Basename of DLL file
+                FILE   => $args{dl_file},     # Dir + Basename of symlist file
+                VERSION =>
+                    (defined $args{dl_version} ? $args{dl_version} : '0.0'),
+            );
+
+            # Mksymlists will create one of these files
+            return grep -e, map "$args{dl_file}.$_", qw(ext def opt);
+        }
     }
     {
 
         sub ACTION_code {
-            require Alien::FLTK2;    # Should be installed by now
+            require Alien::FLTK2;             # Should be installed by now
             my ($self, $args) = @_;
             my $AF = Alien::FLTK2->new();
             my $CC = My::ExtUtils::CBuilder->new();
@@ -67,19 +90,51 @@ package inc::MBX::FLTK;
                 }
                 map { abs2rel($_) if -f } @obj;
             }
-        XS: for my $XS (@xs) {
-                my $cpp = _xs_to_cpp($self, $XS)
-                    or do { printf 'Cannot Parse %s', $XS; exit 0 };
+            my @cpp;
+        XS: for my $XS ((sort { lc $a cmp lc $b } @xs)) {
+                push @cpp, _xs_to_cpp($self, $XS)
+                    or exit !printf 'Cannot Parse %s', $XS;
+                {
+                    open(my ($FH), '<', $XS) || return;
+                    sysread($FH, my ($content), -s $FH) == -s $FH or return;
+                    my @packages = $content =~ m[^MODULE\s*?=\s*?(\S+).+?$]mg;
+                    close $FH;
+                    for my $package (@packages) { $dl_funcs{$package} = []; }
+                }
+            }
+            if (!$self->up_to_date(\@xs, 'xs/include/FLTK_pm_boot.h')) {
+
+                # Generate xs/includes/boot.h
+                print q[Generating 'xs/include/FLTK_pm_boot.h'... ];
+                open(my ($BOOT_H), '>', 'xs/include/FLTK_pm_boot.h')
+                    || die 'Failed to create boot header: ' . $!;
+                my @lines = sprintf "/* Autogenerated by %s:%d at %s */;",
+                    __FILE__, __LINE__, scalar gmtime;
+                push @lines, 'void reboot() {';
+                push @lines, qq[    call( cvrv, "$_" );]
+                    for sort { uc $a cmp uc $b }
+                    grep     {m[^FLTK::]} keys %dl_funcs;
+                push @lines, '}';
+                syswrite($BOOT_H, join "\r\n", @lines);
+                close $BOOT_H;
+                $self->add_to_cleanup('xs/include/FLTK_pm_boot.h');
+                print "okay\n";
+            }
+        CPP: for my $cpp (@cpp) {
                 if ($self->up_to_date($cpp, $CC->object_file($cpp))) {
                     push @obj, $CC->object_file($cpp);
-                    next XS;
+                    next CPP;
                 }
-                push @obj,
-                    $CC->compile('C++'        => 1,
-                                 source       => $cpp,
-                                 include_dirs => [$AF->include_dirs()],
-                                 extra_compiler_flags => [$AF->cxxflags()]
-                    );
+                local $CC->{'quiet'} = 1;
+                printf q[Building '%s'... ], $cpp;
+                my $obj = $CC->compile(
+                                     'C++'        => 1,
+                                     source       => $cpp,
+                                     include_dirs => [$AF->include_dirs()],
+                                     extra_compiler_flags => [$AF->cxxflags()]
+                );
+                printf "%s\n", $obj ? 'okay' : 'failed';    # XXX - exit?
+                push @obj, $obj;
             }
             make_path(catdir(qw[blib arch auto FLTK]),
                       {verbose => !$self->quiet(), mode => 0777});
@@ -119,17 +174,22 @@ package inc::MBX::FLTK;
                 catdir('xs');
             $self->add_to_cleanup($cpp);
             return $cpp
-                if $self->up_to_date([@xsi, $xs, catdir('xs', $typemap)],
-                                     $cpp);
-            printf '%s -> %s (%s)... ', $xs, $cpp, $typemap;
+                if $self->up_to_date([@xsi, $xs,
+                                      rel2abs(catdir('xs', $typemap))
+                                     ],
+                                     $cpp
+                );
+            printf q"Parsing '%s' into '%s' w/ '%s'... ", $xs, $cpp, $typemap;
+            local @ExtUtils::ParseXS::BootCode = ();
 
-            if (ExtUtils::ParseXS->process_file(filename    => $xs,
-                                                output      => $cpp,
-                                                'C++'       => 1,
-                                                hiertype    => 1,
-                                                typemap     => $typemap,
-                                                prototypes  => 1,
-                                                linenumbers => 0
+            if (ExtUtils::ParseXS->process_file(
+                                   filename => $xs,
+                                   output   => $cpp,
+                                   'C++'    => 1,
+                                   hiertype => 1,
+                                   typemap => rel2abs(catdir('xs', $typemap)),
+                                   prototypes  => 1,
+                                   linenumbers => 1
                 )
                 )
             {   print "okay\n";
@@ -156,10 +216,9 @@ package inc::MBX::FLTK;
         {
 
             package Pod::APIDoc::FLTK;
-            use Pod::Parser;
+            use base 'Pod::Parser';
             use strict;
             use warnings;
-            our @ISA = qw[Pod::Parser];
             my ($package, $function, $current) = ('FLTK');
 
             sub command {
@@ -230,8 +289,9 @@ package inc::MBX::FLTK;
                 if ($line =~ m[^INCLUDE: (.+\.xsi?)]) {
                     $self->parse_from_file('xs/' . $1);
                 }
-                elsif ($line =~ m[^MODULE\s*=\s*FLTK\s+PACKAGE\s*=\s*(.+)]) {
-                    $package = $1;
+                elsif ($line
+                      =~ m[^MODULE\s*=\s*FLTK(?:::\w+)?\s+PACKAGE\s*=\s*(.+)])
+                {   $package = $1;
                 }
                 return $line;
             }
@@ -255,9 +315,14 @@ package inc::MBX::FLTK;
             my ($self) = @_;
             my $parser = Pod::APIDoc::FLTK->new();
             $parser->parseopts(-want_nonPODs => 1);
-            print 'Parsing XS files for documentation... ';
-            $parser->parse_from_file('xs/FLTK.xs');
-            print "okay\nGenerating documentation... ";
+            my @xs;
+            find(sub { push @xs, $File::Find::name if m[.+\.xs$]; }, 'xs');
+            for my $xs (sort { lc $a cmp lc $b } @xs) {
+                printf q[Parsing '%s' for documentation... ], $xs;
+                $parser->parse_from_file($xs);
+                print "\n";
+            }
+            print "Generating documentation... ";
             for my $package (sort keys %{$parser->{'apidoc_modules'}}) {
                 my $file = './blib/lib/' . $package . '.pod';
                 $file =~ s|::|/|g;
@@ -432,15 +497,15 @@ package inc::MBX::FLTK;
 
 =pod
 
-=for $Rev: 0320651 $
+=for $Rev: a42720f $
 
-=for $Revision: 0320651a3a52bbe39622e67f8bc20dcc04f3ae62 $
+=for $Revision: a42720f2664562bb3248006d107f257f3093c432 $
 
-=for $Date: 2010-09-17 21:49:10Z (Fri, 17 Sep 2010) $ | Last $Modified: 4 hours ago $
+=for $Date: 2010-09-25 05:55:13Z (Sat, 25 Sep 2010) $ | Last $Modified: 15 minutes ago $
 
 =for $URL: http://github.com/sanko/perl-fltk2/raw/master/inc/MBX/FLTK.pm $
 
-=for $ID: FLTK.pm 0320651 2010-09-17 21:49:10Z sanko@cpan.org $
+=for $ID: FLTK.pm a42720f 2010-09-25 05:55:13Z sanko@cpan.org $
 
 =for author Sanko Robinson <sanko@cpan.org> - http://sankorobinson.com/
 
